from . import _generate_function

Abs = _generate_function('ABS')
ACos = _generate_function('ACOS')
ASin = _generate_function('ASIN')
ATan = _generate_function('ATAN')
ATan2 = _generate_function('ATAN2')
Ceil = _generate_function('CEIL')
Cos = _generate_function('COS')
CumulativeSum = _generate_function('CUMULATIVE_SUM')
Derivative = _generate_function('DERIVATIVE')
Difference = _generate_function('DIFFERENCE')
Elapsed = _generate_function('ELAPSED')
Exp = _generate_function('EXP')
Floor = _generate_function('FLOOR')
Histogram = _generate_function('HISTOGRAM')
Ln = _generate_function('LN')
Log = _generate_function('LOG')
Log2 = _generate_function('LOG2')
Log10 = _generate_function('LOG10')
MovingAverage = _generate_function('MOVING_AVERAGE')
NonNegativeDerivative = _generate_function('NON_NEGATIVE_DERIVATIVE')
NonNegativeDifference = _generate_function('NON_NEGATIVE_DIFFERENCE')
Pow = _generate_function('POW')
Round = _generate_function('ROUND')
Sin = _generate_function('SIN')
Sqrt = _generate_function('SQRT')
Tan = _generate_function('TAN')
