from . import _generate_function

Count = _generate_function('COUNT')
Distinct = _generate_function('DISTINCT')
Integral = _generate_function('INTEGRAL')
Mean = _generate_function('MEAN')
Median = _generate_function('MEDIAN')
Mode = _generate_function('MODE')
Spread = _generate_function('SPREAD')
StdDev = _generate_function('STDDEV')
Sum = _generate_function('SUM')
