from . import _generate_function, _generate_function_with_param

Bottom = _generate_function_with_param('BOTTOM')
First = _generate_function('FIRST')
Last = _generate_function('LAST')
Max = _generate_function('MAX')
Min = _generate_function('MIN')
Percentile = _generate_function_with_param('PERCENTILE')
Sample = _generate_function_with_param('SAMPLE')
Top = _generate_function_with_param('TOP')
